C ESA (C) 2000-2018 
C
C This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
C
C    SAS is free software: you can redistribute it and/or modify
C    it under the terms of the GNU General Public License as published by
C    the Free Software Foundation, either version 3 of the License, or
C    (at your option) any later version.
C
C    SAS is distributed in the hope that it will be useful,
C    but WITHOUT ANY WARRANTY; without even the implied warranty of
C    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
C    GNU General Public License for more details.
C
C    You should have received a copy of the GNU General Public License
C    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
      FUNCTION GAMMP (A,X)
C
CC  returns the negative logarithm of the 
CC  incomplete gamma function P(A,X) 
C
C************************ FFORM VERSION 1.1 ************  7-NOV-89 16:16
C
CA  author : S.Doebereiner date: 11-SEP-1996
C
CT  status: not tested
C
C   general description 
CG  Returns the incomplete gamma function P(A,X) 
CG   in the maximum-likelihood representation -ln(P) 
CG  A and X must be >0 !
CG  Modified routines GCF and GSER from Press et al., Numerical Recipes, 
CG   6.2, p.162
CG
C
C   call_var.          type I/O description 
CP  A                   R4  I   argument
CP  X                   R4  I   argument 
CP  GAMMP               R4    O -ln(P(A,X))
C
C   include_block name          description 
CI  RCOMMON:CGENL.CMN  general parameter common block
CI                              
C
C   routines_called    type     description 
CR  GAMMLN              SR      returns ln of the gamma function  
CR  WRFLAG              SR      writes to FLAG output stream
CR  WRERR               SR      write error message
C
C   extensions/system calls     description 
CX 
C
C***********************************************************************
C
C   variables   meaning
C
C
      IMPLICIT NONE
c     INCLUDE 'include/cgenl.cmn'
C
      CHARACTER*8   RTNAME
      REAL*4        GAMMP 
      REAL          A,X,EPS,GLN,GOLD
      REAL          A0,A1,B0,B1,FAC,AN,ANA,ANF,G,GAMMLN
      REAL          AP,SUM,DEL,GAMSER,GAMMCF
      INTEGER       ITMAX,N,IERR
C
      SAVE
C
      DATA          RTNAME /'GAMMP'/
      DATA          ITMAX /1000/
      DATA          EPS  /3.E-7/                
C
      IERR = 0
      GAMMP = 0.
C
      IF (X.LT.A+1) THEN
        GLN=GAMMLN(A)
        AP=A
        SUM=1./A
        DEL=SUM
        DO 111 N=1,ITMAX
          AP=AP+1.
          DEL=DEL*X/AP
          SUM=SUM+DEL
          IF (ABS(DEL).LT.ABS(SUM)*EPS) GO TO 101
  111   CONTINUE
        IERR = 1 
c        CALL WRERR(RTNAME,IERR,'A too large, ITMAX too small ')
C
C  101   GAMSER = SUM * EXP( -X + A*LOG(X) - GLN)
C        GAMMP = -ALOG(GAMSER)
  101   GAMMP = -ALOG(SUM) + X - A*ALOG(X) + GLN      
      ELSE
        GLN=GAMMLN(A)
        GOLD=0.
        A0=1.
        A1=X
        B0=0.
        B1=1.
        FAC=1.
        DO 211 N=1,ITMAX
          AN=FLOAT(N)
          ANA=AN-A
          A0=(A1+A0*ANA)*FAC
          B0=(B1+B0*ANA)*FAC
          ANF=AN*FAC
          A1=X*A0+ANF*A1
          B1=X*B0+ANF*B1
          IF (A1.NE.0) THEN
            FAC=1./A1
            G=B1*FAC
            IF (ABS((G-GOLD)/G).LT.EPS) GO TO 201
            GOLD=G
          ENDIF
  211   CONTINUE
        IERR = 1 
c       CALL WRERR(RTNAME,IERR,'A too large, ITMAX too small ')
C
  201   GAMMCF = EXP(-X + A*ALOG(X) - GLN) * G
        GAMMP = -ALOG(1.-GAMMCF)
C
      ENDIF 
C
      RETURN
      END
